@@ -1,25 +1,40 @@
 use std::{
     collections::BTreeMap,
     path::{Path, PathBuf},
-    sync::Mutex,
 };
 
 pub use bstr;
 use bstr::{BStr, ByteSlice};
 use nom::error::VerboseError;
 use once_cell::sync::Lazy;
+use parking_lot::Mutex;
 pub use tempfile;
 
 static SCRIPT_IDENTITY: Lazy<Mutex<BTreeMap<PathBuf, u32>>> = Lazy::new(|| Mutex::new(BTreeMap::new()));
 
+pub fn run_git(working_dir: &Path, args: &[&str]) -> std::io::Result<std::process::ExitStatus> {
+    std::process::Command::new("git")
+        .current_dir(working_dir)
+        .args(args)
+        .status()
+}
+
 pub fn hex_to_id(hex: &str) -> git_hash::ObjectId {
     git_hash::ObjectId::from_hex(hex.as_bytes()).expect("40 bytes hex")
 }
 
-pub fn fixture_path(path: impl AsRef<str>) -> PathBuf {
+pub fn fixture_path(path: impl AsRef<Path>) -> PathBuf {
     PathBuf::from("tests").join("fixtures").join(path.as_ref())
 }
-pub fn scripted_fixture_repo_read_only(script_name: &str) -> std::result::Result<PathBuf, Box<dyn std::error::Error>> {
+pub fn fixture_bytes(path: impl AsRef<Path>) -> Vec<u8> {
+    match std::fs::read(fixture_path(path.as_ref())) {
+        Ok(res) => res,
+        Err(_) => panic!("File at '{}' not found", path.as_ref().display()),
+    }
+}
+pub fn scripted_fixture_repo_read_only(
+    script_name: impl AsRef<Path>,
+) -> std::result::Result<PathBuf, Box<dyn std::error::Error>> {
     scripted_fixture_repo_read_only_with_args(script_name, None)
 }
 
@@ -35,11 +50,16 @@ pub fn scripted_fixture_repo_writable_with_args(
 ) -> std::result::Result<tempfile::TempDir, Box<dyn std::error::Error>> {
     let ro_dir = scripted_fixture_repo_read_only_with_args(script_name, args)?;
     let dst = tempfile::TempDir::new()?;
+    copy_recursively_into_existing_dir(&ro_dir, dst.path())?;
+    Ok(dst)
+}
+
+pub fn copy_recursively_into_existing_dir(src_dir: impl AsRef<Path>, dst_dir: impl AsRef<Path>) -> std::io::Result<()> {
     fs_extra::copy_items(
-        &std::fs::read_dir(ro_dir)?
+        &std::fs::read_dir(src_dir)?
             .map(|e| e.map(|e| e.path()))
             .collect::<Result<Vec<_>, _>>()?,
-        dst.path(),
+        dst_dir,
         &fs_extra::dir::CopyOptions {
             overwrite: false,
             skip_exist: false,
@@ -49,19 +69,19 @@ pub fn scripted_fixture_repo_writable_with_args(
         },
     )
     .map_err(|err| std::io::Error::new(std::io::ErrorKind::Other, err))?;
-    Ok(dst)
+    Ok(())
 }
 
 /// Returns the directory at which the data is present
 pub fn scripted_fixture_repo_read_only_with_args(
-    script_name: &str,
+    script_name: impl AsRef<Path>,
     args: impl IntoIterator<Item = &'static str>,
 ) -> std::result::Result<PathBuf, Box<dyn std::error::Error>> {
     let script_path = fixture_path(script_name);
 
     // keep this lock to assure we don't return unfinished directories for threaded callers
     let args: Vec<String> = args.into_iter().map(Into::into).collect();
-    let mut map = SCRIPT_IDENTITY.lock().unwrap();
+    let mut map = SCRIPT_IDENTITY.lock();
     let script_identity = map
         .entry(args.iter().fold(script_path.clone(), |p, a| p.join(a)))
         .or_insert_with(|| {
@@ -74,11 +94,7 @@ pub fn scripted_fixture_repo_read_only_with_args(
             crc_digest.finalize()
         })
         .to_owned();
-    let script_result_directory = fixture_path(
-        Path::new("generated")
-            .join(format!("{}", script_identity))
-            .to_string_lossy(),
-    );
+    let script_result_directory = fixture_path(Path::new("generated-do-not-edit").join(format!("{}", script_identity)));
     if !script_result_directory.is_dir() {
         std::fs::create_dir_all(&script_result_directory)?;
         let script_absolute_path = std::env::current_dir()?.join(script_path);
